#ifndef SERIALPORT_H
#define SERIALPORT_H
#include <QMap>
#include <QObject>
#include <QSerialPort>
#include <QTimer>

class SerialPortTool : public QObject {
    Q_OBJECT

public:
    explicit SerialPortTool(QObject* parent = 0);
    ~SerialPortTool();
    void open();
    void close();
public slots:
    void onReset();
    void onPlaybackStart();
    void onPlaybackEnd();
    void onRecordStart();
    void onRecordEnd();
    void onPlayPause();
    void onPlayResume();
    void onDiskWillFull();
    void onDiskNotFull();
    void onPowerOn();
    void onPowerOff();
    void onFoward();
    void onBack();
    void onLoopOn();
    void onLoopOff();
    void onMainChannelOn();
    void onMainChannelOff();
    void onSecondaryChannelOn();
    void onSecondaryChannelOff();
    void onMainChannelHDMI();
    void onMainChannelVGA();
    void onSecondaryChannelHDMI();
    void onSecondaryChannelVGA();

private slots:
    void onSerialError(QSerialPort::SerialPortError error);
    void onTimeout();
    void onReayRead();

signals:
    void btnMenuClicked();
    void btnUpClicked();
    void btnDownClicked();
    void btnLeftClicked();
    void btnRightClicked();
    void btnConfirmClicked();
    void btnReturnClicked();

private:
    enum CommandType {
        Reset,
        PlaybackStart,
        PlaybackEnd,
        RecordStart,
        RecordEnd,
        PlayPause,
        PlayResume,
        DiskWillFull,
        DiskNotFull,
        PowerOn,
        PowerOff,
        Forward,
        Back,
        LoopOn,
        LoopOff,
        MainChannelOn,
        MainChannelOff,
        SecondaryChannelOff,
        SecondaryChannelOn,
        MainChannelHDMI,
        MainChannelVGA,
        SecondaryChannelHDMI,
        SecondaryChanneVGA
    };

    QSerialPort* serialPort; // 串口
    QMap<CommandType, QByteArray> cmdMap; // 指令列表
    QTimer* timer;
};

#endif // SERIALPORT_H
